var supportSVG = checkSVG();
function get(prop) {
  return function(d) {
    return d[prop];
  };
}

var inherit = (function() {
  var F = function() {};
  return function(C, P) {
    F.prototype = P.prototype;
    C.prototype = new F();
    C.uber = P.prototype;
    C.prototype.constructor = C;
  }
}());

function BasicChart(options) {
  this.options = options;
  this.thousandSeparator = this.options.thousandSeparator === undefined ? " " : this.options.thousandSeparator;
  this.decimalSeparator = this.options.decimalSeparator === undefined ? "," : this.options.decimalSeparator;
  
  this.data = options.data;
  this.sorting = options.sorting || "descending";
  // that's a hack
  if (options.width > 580) options.width = 580;
  this.width = options.width;
  this.height = options.height;
  this.headerHeight = options.headerHeight || 50;
  this.graphHeight = options.graphHeight || this.height;
  this.legendHeight = 300;
  this.graphWidth = options.graphWidth || options.width || 300;
  this.headerPadding;
  this.graphPadding = options.graphPadding || {
    top: 40,
    right: 65,
    bottom: 40,
    left: 65
  };
  this.bottomLinePadding = options.bottomLinePadding || {
    left: 10,
    right: 30
  };
  this.legendPadding = options.legendPadding || {
    top: 15,
    left: 0,
    right: 20,
    bottom: 5
  };
  this.markHeight = 15;
  this.legendType = options.legendType === undefined ? "block" : options.legendType;
  this.graphArea = null;
  this.colors = ["#465160", "#8a919e", "#bbbfca", "#9aa16d", "#5179a7", "#d55949", "#aeb692", "#6086ae", "#cdd2b9", "#93aecf", "#bbbfca", "#d65e4a", "#d96a53", "#df856c", "#e8a78f", "#f3d0c0"];
}

BasicChart.prototype.drawBase = function() {
  var that = this;
  var data = this.data;
  var options = this.options;
  var container = d3.selectAll(options.container);
  var fontSize = 15;
  var width = this.width;
  var height = this.height;
  var markHeight = this.markHeight;
  var svg = this.svg = container.insert("svg", ":first-child")
    .attr("xmlns", "http://www.w3.org/2000/svg")
    .attr("version", "1.1")
    .attr("xmlns:xmlns:xlink", "http://www.w3.org/1999/xlink")
    .attr("width", width)
    .style("font-size", fontSize + "px")
    .style("font-family", "rail, Helvetica, sans-serif")
    .style("font-style", "normal")


  var field = this.field = svg.append("g").classed("field", true);
  var header = this.header = field.append("g").classed("chart-header", true);
  var graphArea = this.graphArea = field.append("g").classed("graph-area", true);
  var legend = this.legend = field.append("g").classed("chart-legend", true)
    .style("font-family", "Helvetica, Arial, sans-serif")

  svg.append("line")
    .attr("x1", 0)
    .attr("x2", width)
    .attr("y1", 0)
    .attr("y2", 0)
    .attr("stroke-width", 2)
    .attr("stroke", "#616c7b")

  var title = header.append("text")
    .attr("fill", "#616c7b")
    .attr("font-size", 12 / 15 + "em")
    .attr("height", 300)
    .attr("y", "2em")
    .style("text-transform", "uppercase")
  title.selectAll("tspan")
    .data(data.chartTitle.split("\n")).enter()
    .append("tspan")
    .attr("dy", function(d, i) {
      return i == 0 ? 0 : "1.5em"
    })
    .text(String)
    .attr("x", 0)

  this.headerBox = header.node().getBBox();
  this.headerHeight = this.headerBox.height + this.headerBox.y; // +font size
  svg.append("line").attr("x1", this.bottomLinePadding.left).attr("x2", width - this.bottomLinePadding.right)
    .attr("y1", this.headerHeight + this.graphHeight)
    .attr("y2", this.headerHeight + this.graphHeight)
    .attr("stroke-width", 2)
    .attr("stroke", "#616c7b")
    .attr("stroke-dasharray", "2 4")

  this.graphArea.attr("transform", "translate(0, " + (this.headerHeight) + ")")

  var legendHeight = this.legendHeight = 0;
  //  adding legend
  if (data.layers.length > 1) {
    var lines = [];
    lines[0] = [];
    var curLine = 0;
    var itemMargin = 45;
    var lineHeight = 30;
    var calcLineWidth = function(line) {
      return d3.sum(line.map(function(d) {
        return d.getBBox().width + itemMargin;
      }));
    }
    var calcLineHeight = function(line) {
      return d3.max(line.map(function(d) {
        return d.getBBox().height;
      }));
    }

     if (that.sorting) {
      that.data.layers.sort(function(a, b) {
        return d3[that.sorting](a.value, b.value);
      })
      this.titles = this.data.layers.map(function(d) {
        return d.title
      });
    }

    var legend = this.legend
      .attr("transform", "translate(" + (this.legendPadding.left) + "," + (this.headerHeight + this.graphHeight + this.legendPadding.top) + ")")
      .attr("font-size", (11 / 15) + "em")

    if (this.legendType == "inline") {
      var items = this.legendItems = legend.selectAll("g.legend-item").data(this.titles.filter(function(d) {
        return d.toString().length > 0
      }), function(d) {
        return d;
      })
        .enter().append("g")
        .attr("class", "legend-item")
      // .attr("height", markHeight)
      .on("mouseover", function(d) {
        that.showOnly([d]);
      })
        .on("mouseout", function() {
          that.showOnly(that.titles);
        })
      items.append("text")
        .attr("fill", "#4b4b4b")
        .attr("dx", markHeight * 1.5)
        .attr("y", "-.5em")
        .attr("transform", "translate(0," + markHeight + ")")
        .selectAll("tspan").data(function(d) {
          return d.toString().split("\n")
        }).enter()
        .append("tspan")
        .style("dominant-baseline", "central")
        .text(String)
        .attr("dy", function(d, i) {
          return i == 0 ? 0 : "1em";
        })
        .attr("x", function(d, i) {
          return i == 0 ? 0 : markHeight * 1.5
        })

      var legendHeight = 0;
      items.each(function(d, i) {
        var dx = 0,
          dy = 0;
        var cw = this.getBBox().width + itemMargin;
        lines[curLine].push(this);
        var lw = calcLineWidth(lines[curLine]);

        if (lw > width) {
          legendHeight += calcLineHeight(lines[curLine]);
          lines[++curLine] = [];
          lines[curLine].push(lines[curLine - 1].splice(lines[curLine - 1].length - 1, 1)[0]);
        }
        dx = calcLineWidth(lines[curLine]) - cw;
        dy = legendHeight + lineHeight * .4 * curLine;
        d3.select(this).attr("transform", "translate(" + dx + "," + dy + ")")
      })

      var legendMarks = items.insert("rect", "text")
        .attr("width", markHeight)
        .attr("height", markHeight)
      // .attr("transform","translate(0,0)")
      .attr("fill", function(d, i) {
        return that.getColor(d);
      })
        .attr("stroke", function(d, i) {
          return that.getColor(d);
        })
        .attr("stroke-width", 0);
    }

    if (this.legendType == "block") {
      var items = this.legendItems = legend.selectAll("g.legend-item")
        .data(this.titles, function(d) {
          return d;
        })
        .enter().append("g")
        .attr("class", "legend-item")
      // .attr("height", markHeight)
      .on("mouseover", function(d) {
        that.showOnly([d]);
      })
        .on("mouseout", function() {
          that.showOnly(that.titles);
        })
      items.append("rect")
        .attr("width", markHeight)
        .attr("height", markHeight)
        .attr("fill", function(d) {
          return that.getColor(d);
        })
      items.append("text")
        .style("dominant-baseline", "central")
        .attr("fill", "#4b4b4b")
      // .attr("y", ".5em")
      .attr("transform", "translate(0, " + 0 + ")")
        .attr("dx", markHeight * 1.5)
        .attr("y", "1em")
        .selectAll("tspan").data(function(d) {
          return d.toString().split("\n")
        }).enter()
        .append("tspan")
        .text(String)
        .attr("dy", function(d, i) {
          return i == 0 ? 0 : "1.5em";
        })
        .attr("x", function(d, i) {
          return i == 0 ? 0 : markHeight * 1.5
        })
      var itemHeight = 0;
      items.attr("transform", function(d) {
        var dy = itemHeight;
        itemHeight += this.getBBox().height + markHeight * .6;
        return "translate(0, " + dy + ")"
      })
    }

  }
  this.legendHeight = this.legend.node().getBBox().height;

  this.svgHeight = this.headerHeight + this.graphHeight + this.legendHeight + this.legendPadding.top + this.legendPadding.bottom;
  svg.attr("height", (this.svgHeight))

    if (this.data.noteMarks) {
      title.selectAll(".chart-note-mark").data(this.data.noteMarks).enter()
      .append("tspan").text(function(d,i){
        return i > 0 ? ", "+d : " "+d;
      }).attr("font-size", ".8em").attr("class", "chart-note-mark").attr("baseline-shift", "super")
    }

    if (!this.data.notes || !this.data.notes[0]) return;

    container.selectAll(".chart-note").data(this.data.notes).enter()
      .append("div").attr("class", "chart-note")
      .html(function(d,i){
        return "<sup>"+data.noteMarks[i]+"</sup>"+d;
      })
    


}

function StackChart(options) {
  BasicChart.apply(this, arguments);
  this.base = new BasicChart(options);
  this.options = options;
  this.data = options.data;
  this.outerWidth = this.width - this.graphPadding.left - this.graphPadding.right;
  this.outerHeight = this.height - this.graphPadding.top - this.graphPadding.bottom;
  this.orientation = options.orientation || "horizontal";
  this.calcTotal = options.calcTotal === undefined ? true : options.calcTotal;
  this.format = options.format === undefined ? ",.1f" : options.format;
  this.showPercents = options.showPercents == undefined ? true : options.showPercents;
  var graphPadding = {
    top: 20,
    left: 0,
    right: 50,
    bottom: 5
  };
  if (this.orientation == "vertical") {
    graphPadding.left = 15;
    graphPadding.right = 15;
  }
  this.graphPadding = options.graphPadding || graphPadding;
  this.graphHeight = this.height;
  var legendPadding = {
    top: 15,
    left: 0,
    right: 20,
    bottom: 5
  };
  if (this.orientation == "vertical") legendPadding.top = 40;
  this.legendPadding = options.legendPadding || legendPadding;
  this.bottomLinePadding = options.bottomLinePadding || {
    left: 0,
    right: 0
  };
  this.titles = this.data.layers.map(function(d) {
    return d.title
  });
  this.barWidth = options.barWidth === undefined ? 20 : options.barWidth;
  this.textHeight = 15;
  return this;
}

StackChart.prototype.getColor = function(title) {
  // TODO: make this with ordinal scale
  var colors = this.colors;
  var titles = this.titles;
  var res = null;
  titles.forEach(function(t, i) {
    if (t == title)
      res = colors[i % colors.length];
    return;
  })
  return res;
}

StackChart.prototype.showOnly = function(keys) {
  var duration = this.options.duration === undefined ? 600 : this.options.duration;
  var x = this.x;
  var y = this.y;
  var that = this;
  var field = this.field;
  var keyIndex = this.titles.indexOf(keys[0]);
  var newLayers = this.layersData.filter(function(d) {
    return keys.some(function(k) {
      return k == d.key
    })
  });
  var vals = newLayers.map(function(d) {
    return d.value
  });

  var totals = this.dates.map(function(d, i) {
    return {
      title: d,
      value: d3.sum(newLayers, function(l) {
        return l.value[i].y;
      })
    }
  })
  var totalShares = totals.map(function(d, i){
    var value = d.value/that.totals[i].value;
    return {
      title: d.title,
      value: value,
      absoluteValue: d.value
    }
  })
  // s
  


  if (that.options.orientation == "vertical") {
    y.domain([0, d3.max(d3.merge(vals), function(d) {
      return keys.length == 1 ? d.y : d.y0 + d.y;
    })]);
    var layersGroup = field.selectAll(".layer").data(newLayers, function(d) {
      return d.key
    });
    var exit = layersGroup.exit()
    exit.selectAll("rect")
      .transition()
      .duration(duration)
      .attr("transform", function(d, i) {
        var index = that.titles.indexOf(d.key);
        var dy = index > keyIndex ? y.range()[1] - y(totals[i].value) : y.range()[1];
        return "translate(" + (x(d.x)) + ", " + (dy) + ") scale(1, 0)";
      });

    var enter = layersGroup.enter()
    var update = layersGroup
    update.selectAll("rect")
      .transition()
      .duration(duration)
      .attr("height", function(d) {
        return y(d.y)
      })
      .attr("transform", function(d) {
        var dy = keys.length > 1 ? y.range()[1] - y(d.y0) - y(d.y) : y.range()[1]-y(d.y)
        return "translate(" + (x(d.x)) + ", " + (dy) + ")";
      });

    update.selectAll("text.label")
      .transition()
      .duration(duration)
      .attr("transform", function(d) {
        var dy = keys.length > 1 ? y(d.y0) + y(d.y) * .5 : y(d.y) * .5;
        return "translate(" + dx + ", " + (y(d.x) + y.rangeBand() * .5) + ")";
      })
      .attr("opacity", 1)

    field.selectAll(".total-val")
      .data(totals)
      .transition()
      .duration(duration)
      .attr("transform", function(d, i) {
        var dy = -y(d.value) + y.range()[1];
        return "translate(" + (x(d.title)) + "," + (dy) + ")";
      })
      .tween("fill", function(d, i) {
        var ip = d3.interpolateRgb(d3.select(this).attr("fill"), keys.length > 1 ? "#425563" : that.getColor(keys[0]));
        return function(t) {
          d3.select(this).attr("fill", ip(t));
        };
      })
      .tween("text", function(d, i) {
        var ip = d3.interpolateNumber(this.textContent.split(that.thousandSeparator).join("").split(that.decimalSeparator).join("."), d.value);
        return function(t) {
          res = ip(t);
          res = d3.format(that.format)(res);
          res = res.split(",").join(that.thousandSeparator);
          res = res.split(".").join(that.decimalSeparator);
          this.textContent = res;
        };
      })


  }
  else {
    if (that.options.type == "stack") {
      x.domain([0, d3.max(d3.merge(vals), function(d) {
        return keys.length == 1 ? d.y : d.y0 + d.y;
      })]);
      var layersGroup = field.selectAll(".layer").data(newLayers, function(d) {
        return d.key
      });
      var exit = layersGroup.exit()
      exit.selectAll("rect")
        .transition()
        .duration(duration)
        .attr("transform", function(d, i) {
          var index = that.titles.indexOf(d.key);
          var dx = index > keyIndex ? x(totals[i].value) : 0;
          return "translate(" + (dx) + ", " + (y(d.x) + that.barWidth) + ") scale(0, 1)";
        });

      var enter = layersGroup.enter()
      var update = layersGroup
      update.selectAll("rect")
        .transition()
        .duration(duration)
        .attr("width", function(d) {
          return x(d.y)
        })
        .attr("transform", function(d) {
          var dx = keys.length > 1 ? x(d.y0) : 0
          return "translate(" + dx + ", " + (y(d.x) + that.barWidth) + ")";
        });

      update.selectAll("text.label")
        .transition()
        .duration(duration)
        .attr("transform", function(d) {
          
          var dx = keys.length > 1 ? x(d.y0) + x(d.y) * .5 : x(d.y) * .5;
          return "translate(" + dx + ", " + (y(d.x) + y.rangeBand() * .5) + ")";
        })
        .attr("opacity", 1)

      field.selectAll(".total-val")
        .data(totals)
        .transition()
        .duration(duration)
        .attr("transform", function(d, i) {
          var dx = x(d.value);
          return "translate(" + (dx) + "," + (y(d.title) + that.barWidth * 1.5) + ")";
        })
        .tween("fill", function(d, i) {
          var ip = d3.interpolateRgb(d3.select(this).attr("fill"), keys.length > 1 ? "#425563" : that.getColor(keys[0]));
          return function(t) {
            d3.select(this).attr("fill", ip(t));
          };
        })
        .tween("text", function(d, i) {
          var ip = d3.interpolateNumber(this.textContent.split(that.thousandSeparator).join("").split(that.decimalSeparator).join("."), d.value);
          return function(t) {
            res = ip(t);
            res = d3.format(that.format)(res);
            res = res.split(",").join(that.thousandSeparator);
            res = res.split(".").join(that.decimalSeparator);
            this.textContent = res;
          };
        })
        if (that.showPercents) {
          var percents = field.select(".stack-graph").selectAll("text.percent-val").data(totalShares)
          // percents.exit().transition().attr("opacity", 0)
          percents.transition().attr("opacity", 0).transition().attr("opacity", function(d){
            return keys.length > 1 ? 0 : 1;
          }).text(function(d){
            var labelWidth = this.getBBox().width;
            var formatted = d3.format(".1%")(d.value);
            if (x(d.absoluteValue) < labelWidth+13) {
              formatted = "("+formatted+")";
            }
            return d.value == 0 ? "" : formatted;
          })
          .attr("transform", function(d,i){
            var labelWidth = this.getBBox().width;
            var dx = x.range()[0];
            if (x(d.absoluteValue) < labelWidth+13) {
              dx += labelWidth+x(d.absoluteValue)-10;
            }
            return "translate(" + (dx) + "," + (y(d.title) + y.rangeBand()*.75) + ")";
          }).attr("fill", function(d){
            var labelWidth = this.getBBox().width;
              return (x(d.absoluteValue) < labelWidth+13) ? "#000" : "#fff"
              
          })
          percents.enter().append("text").attr("class", "percent-val").attr("transform", function(d,i){
              return "translate(" + (x.range()[0]) + "," + (y(d.title) + y.rangeBand()*.75) + ")";
            })
          .style("font-size", 13 / 15 + "em")
          .attr("x", "1em")
          .text(function(d){
              return d3.format(".1%")(d.value);
            }).attr("fill", "#fff").attr("opacity", 0)
          .transition().attr("opacity", 1);
        }
        
    }

    if (that.options.type == "group") {
      x.domain([0, d3.max(d3.merge(vals), function(d) {
        return d.y;
      })]);
      // x.range([0, this.width]);
      var layersGroup = field.selectAll(".layer").data(newLayers, function(d) {
        return d.key
      });
      var exit = layersGroup.exit()
      exit.selectAll("rect")
        .transition()
      // .attr("height", 0)
      .attr("opacity", 0)
      // .attr("height", (y.rangeBand()/newLayers.length - 1))
      .attr("transform", function(d, i, j) {
        var dx = x(0);
        var dy = (y(d.x) + (j + 2) * y.rangeBand() / that.titles.length);
        return "translate(" + dx + ", " + dy + ")";
      });

      var enter = layersGroup.enter()

      var update = layersGroup
      update.selectAll("rect").transition()
        .attr("width", function(d) {

          return x(d.y)
        })
        .attr("transform", function(d, i, j) {
          var dx = x(0);
          var dy = (y(d.x) + j * y.rangeBand() / newLayers.length);
          return "translate(" + dx + ", " + dy + ")";
        })
        .attr("height", function(d) {
          return keys.length == 1 ? y.rangeBand() : (y.rangeBand() / that.titles.length - 1)
        })
        .attr("opacity", 1)

      update.selectAll("text.label")
        .transition()
        .attr("transform", function(d, i, j) {
          var dx = x(d.y) + 5;
          var dy = (y(d.x) + j * y.rangeBand() / newLayers.length) + y.rangeBand() / newLayers.length * .5;

          return "translate(" + dx + ", " + dy + ")";
        })
        .attr("opacity", 1)

      exit.selectAll("text.label")
        .transition()
        .attr("opacity", 0)
      // .attr("transform", function(d) {
      //  var dx = x(d.y)*.5
      //  return "translate(" + dx + ", " + (y(d.x)+y.rangeBand()*.5) + ")";
      // })
    }
  }


  
  var leg = this.legendItems.data(keys, function(d) {
    return d
  });
  var legEnter = leg.enter();
  var legExit = leg.exit();
  leg
    .transition()
    .attr("opacity", 1)
    .selectAll("rect")
    .attr("stroke-width", function() {
      return keys.length == 1 ? 2 : 0;
    })

  legExit
    .transition()
    .attr("opacity", .5)
    .selectAll("rect")
    .attr("stroke-width", 0)


}

StackChart.prototype.drawVertical = function() {
  

  var that = this;
  var data = this.data;
  var stack = this.stack = d3.layout.stack();
  var dates = this.dates = this.data.categories;
  var padding = this.graphPadding;
  var width = this.width;
  var height = this.height;
  var offsetLeft = this.offsetLeft;
  var markHeight = 15; // legend mark height
  var sd = this.layers = this.data.layers.map(function(d, i) {
    return d.values.map(function(v, j) {
      return {
        x: dates[j],
        y: v
      };
    });
  })
  var layers = stack(sd);
  var svg = this.svg;
  var field = this.field;
  var header = this.header;
  var headerHeight = header.node().getBBox().height;
  var graphArea = this.graphArea
  var graph = graphArea.append("g").classed("stack-graph", true)
    .attr("transform", "translate(" + this.graphPadding.left + "," + this.graphPadding.top + ")");
  var gHeight = this.graphHeight - this.graphPadding.top - this.graphPadding.bottom;
  var gWidth = this.graphWidth - this.graphPadding.left - this.graphPadding.right;
  var x = this.x = d3.scale.ordinal();
  var y = this.y = d3.scale.linear();
  var max = d3.max(d3.merge(layers), function(d) {
    return d.y0 + d.y;
  });
  var min = d3.min(d3.merge(layers), function(d) {
    return d.y0 + d.y;
  });
  var maxDomain = max;
  var maxRange = gHeight;
  if (max*min < 0) {
    maxDomain = Math.max(max, -min);
    maxRange = gHeight*.5
  }
  y.domain([0, maxDomain]);
  
  y.range([0, maxRange])
  x.rangeBands([0, gWidth], .5, 0);
  x.domain(dates);

  var xAxis = graph.append("g").attr("class", "x axis")
  xAxis.selectAll(".tick").data(dates)
    .enter().append("text").attr("class", "tick")
    .text(String)
    .attr("transform", function(d) {
      
      return "translate("+x(d)+"," + (y.range()[1]+that.graphPadding.bottom) + ")";
    })
    .attr("y", "1.5em")
    .attr("x", x.rangeBand()*.5)
    .style("font-size", 12 / 15 + "em")
    .attr("text-anchor", "middle")
  // .style("dominant-baseline", "central")
  .attr("fill", "#284b81");

  var layersData = this.layersData = layers.map(function(d, i) {
    return {
      key: that.titles[i],
      value: d
    }
  })

  var layer = graph.selectAll("g.layer")
    .data(layersData, function(d, i) {
      return d.key;
    })
    .enter().append("svg:g")
    .attr("class", function(d, i) {
      return "layer-" + i
    })
    .classed("layer", true)
    .attr("fill", function(d) {
      return that.getColor(d.key);
    })
    .attr("stroke", function(d) {
      return that.getColor(d.key);
    })

    
    var rect = layer.selectAll("rect")
      .data(function(d) {
        return d.value.map(function(v) {
          return {
            x: v.x,
            y: v.y,
            y0: v.y0,
            key: d.key
          };
        });
      })
      .enter().append("svg:rect")
      .attr("height", function(d) {
        
        
        return Math.abs(y(d.y));
      })
      .attr("width", function(d) {
        return x.rangeBand();
      })
      .attr("transform", function(d) {
        var dy = y.range()[1] - (d.y+d.y0 > 0 ? y(d.y+d.y0) : 0)
        return "translate(" + x(d.x) + ", " + dy + ")";
      })
      .attr("stroke-width", 0);

      if (data.layers.length > 1) {
        rect.on("mouseover", function(d) {
          that.showOnly([d.key]);
        }).on("mouseout", function() {
          that.showOnly(that.titles)
        })
      }

    var totals, calcTotals;
    calcTotals = dates.map(function(d, i) {
      return {
        "title": d,
        "value": d3.sum(data.layers, function(l) {
          return l.values[i];
        })
      }
    })
    this.totals = totals = calcTotals;

    var totalText = graph.selectAll(".total-val").data(totals)
      .enter().append("text").attr("class", "total-val")
      .text(function(d, i) {
        var res = d3.format(that.format)(d.value);
        res = res.split(",").join(that.thousandSeparator);
        res = res.split(".").join(that.decimalSeparator);
        return res;
      })
      .attr("transform", function(d, i) {
        
        var dy = -y(d.value) + y.range()[1];
        if (d.value < 0) {
          xAxis.selectAll(".tick").filter(function(d,j){
            return j==i;
          }).attr("y", "-1.5em")
        }
        return "translate(" + (x(d.title)) + "," + (dy) + ")";
      })
      .attr("x", x.rangeBand()*.5)
      .attr("y", function(d){
        return d.value > 0 ? "-.5em" : "1.5em"
      })
      .attr("text-anchor", "middle")
      .style("font-size", 11 / 15 + "em")
      // .style("dominant-baseline", "central")
      .attr("fill", function(d){
        return d.value < 0 ? "#e21a1a" : "#284b81";
      });
}

StackChart.prototype.draw = function() {
  // TODO: stack chart with negative values (multiple)
  this.base.drawBase.call(this);

  if (this.options.orientation == "vertical")  {
    this.drawVertical();
    return;
  }
  var that = this;
  var data = this.data;
  var stack = this.stack = d3.layout.stack();
  var dates = this.dates = this.data.categories;
  var padding = this.graphPadding;
  var width = this.width;
  var height = this.height;
  var offsetLeft = this.offsetLeft;
  var markHeight = 15; // legend mark height
  var sd = this.layers = this.data.layers.map(function(d, i) {
    return d.values.map(function(v, j) {
      return {
        x: dates[j],
        y: v
      };
    });
  })
  var layers = stack(sd);

  var svg = this.svg;

  var field = this.field;

  var header = this.header;
  var headerHeight = header.node().getBBox().height;
  var graphArea = this.graphArea
  var graph = graphArea.append("g").classed("stack-graph", true)
    .attr("transform", "translate(" + this.graphPadding.left + "," + this.graphPadding.top + ")");
  var gHeight = this.graphHeight - this.graphPadding.top - this.graphPadding.bottom;

  var x = this.x = d3.scale.linear();
  var gWidth = width - this.graphPadding.left - this.graphPadding.right;
  var y = this.y = d3.scale.ordinal().rangeBands([0, gHeight], 0, 0);
  y.domain(dates);


  var barWidth = this.barWidth = this.y.rangeBand() * .45;
  
  var max = d3.max(d3.merge(layers), function(d) {
    return d.y0 + d.y;
  });
  var min = d3.min(d3.merge(layers), function(d) {
    return d.y0 + d.y;
  });
  var maxDomain = max;
  var minDomain = 0;
  var minRange = 0;
  var maxRange = gWidth;
  if (max*min < 0) {
    maxDomain = Math.max(max, -min);
    minDomain = -maxDomain;
    // maxRange = gWidth*.5;
    minRange = -gWidth*.5
  }
  x.range([minRange, minRange + maxRange]);
 

  if (this.options.type == "stack") {
    x.domain([minDomain, maxDomain]);
  }
  if (this.options.type == "group") {
    x.domain([0, d3.max(d3.merge(layers), function(d) {
      return d.y;
    })]);
    
    var suby = d3.scale.ordinal().rangeBands([0, y.rangeBand()], .1, .2).domain(this.titles);
    
  }



  var yAxis = graph.append("g").attr("class", "y axis")
  yAxis.selectAll(".tick").data(dates)
    .enter().append("text").attr("class", "tick")
    .text(String)
    .attr("transform", function(d) {
      return "translate(0," + (y(d) + barWidth) + ")";
    })
    .attr("y", -this.y.rangeBand() * .1)
    .style("font-size", 12 / 15 + "em")
  // .style("dominant-baseline", "central")
  .attr("fill", "#284b81");

  var layersData = this.layersData = layers.map(function(d, i) {
    return {
      key: that.titles[i],
      value: d
    }
  })


  // Add a rect for each date.
  if (this.options.type == "stack") {

    var layer = graph.selectAll("g.layer")
      .data(layersData, function(d, i) {
        return d.key;
      })
      .enter().append("svg:g")
      .attr("class", function(d, i) {
        return "layer-" + i
      })
      .classed("layer", true)
      .attr("fill", function(d) {
        return that.getColor(d.key);
      })
      .attr("stroke", function(d) {
        return that.getColor(d.key);
      })
    graph.selectAll("line.group-divider")
      .data(dates).enter().append("line")
      .attr("y1", function(d) {
        return y(d);
      })
      .attr("y2", function(d) {
        return y(d);
      })
      .attr("x1", 0).attr("x1", width)
      .attr("stroke-width", 1)
      .attr("stroke", "#616c7b")
      .attr("stroke-dasharray", "1 2")

    var rect = layer.selectAll("rect")
      .data(function(d) {
        return d.value.map(function(v) {
          return {
            x: v.x,
            y: v.y,
            y0: v.y0,
            key: d.key
          };
        });
      })
      .enter().append("svg:rect")
      .attr("height", barWidth)
      .attr("width", function(d) {
        return Math.abs(x(d.y));
      })
      .attr("transform", function(d) {
        var dx = -x.range()[0]+(d.y+d.y0 > 0 ? x(d.y0) : x(d.y+d.y0));
        return "translate(" + dx + ", " + (y(d.x) + barWidth) + ")";
      })
      .attr("stroke-width", 0);
  }

  if (this.options.type == "group") {
    var groupLayers = this.data.layers.map(function(l)  {
      return {
        title: l.title,
        values: l.values.map(function(v,i){
          return {
            value: v,
            key: dates[i]
          }
        })
      }
    })
    var layer = graph.selectAll("g.layer")
      .data(groupLayers, function(d, i) {
        return d.title;
      })
      .enter().append("svg:g")
      .attr("class", function(d, i) {
        return "layer-" + i
      })
      .classed("layer", true)
      .attr("fill", function(d) {
        return that.getColor(d.title);
      })
      .attr("stroke", function(d) {
        return that.getColor(d.title);
      }).attr("transform", function(d){
        
        return "translate(0,"+suby(d.title)+")"
      })

    var rect = layer.selectAll("rect")
      .data(function(d){
        
        return d.values;
      })
      .enter().append("svg:rect")
      .attr("height", suby.rangeBand())
      .attr("width", function(d) {
        return x(d.value);
      })
      .attr("transform", function(d, i, j) {
        var dx = 0;
        var dy = (y(d.key));
        return "translate(" + dx + ", " + dy + ")";
      })
      .attr("stroke-width", 0);
    var maxWidth = 0;
    var label = layer.selectAll("text.label")
      .data(function(d) {
        return d.value.map(function(v) {
          return {
            x: v.x,
            y: v.y,
            y0: v.y0,
            key: d.key
          };
        });
      })
      .enter().append("text")
      .attr("class", "label")
      .attr("transform", function(d, i, j) {
        var dx = x(d.y) + 5;
        var dy = (y(d.x) + j * y.rangeBand() / layersData.length) + y.rangeBand() / layersData.length * .5;
        var textWidth = this.getBBox().width;
        maxWidth = textWidth > maxWidth ? textWidth : maxWidth;
        return "translate(" + dx + ", " + dy + ")";
      })
      .attr("stroke-width", 0)
      .attr("font-size", 10 / 15 + "em")
      .text(function(d) {
        var res = d3.format(that.format)(d.y);
        res = res.split(",").join(that.thousandSeparator);
        res = res.split(".").join(that.decimalSeparator);
        return res;
      })
      .style("dominant-baseline", "central")
      .attr("text-anchor", "start")
      .attr("fill", "#368cc5")
  }
  if (data.layers.length > 1) {
    rect.on("mouseover", function(d) {
      that.showOnly([d.key]);
    }).on("mouseout", function() {
      that.showOnly(that.titles)
    })
  }
  //  adding total values
  var maxWidth = 0;

  if (this.options.type == "stack") {
    var totals, calcTotals;
    calcTotals = dates.map(function(d, i) {
      return {
        "title": d,
        "value": d3.sum(data.layers, function(l) {
          return l.values[i];
        })
      }
    })
    if (!this.calcTotal)
      totals = data.total.values.map(function(d,i){
        return {
          "title": dates[i],
          "value": d
        }
      });
    else {
      totals = calcTotals;
    }
    this.totals = totals;
    var totalText = graph.selectAll(".total-val").data(calcTotals)
      .enter().append("text").attr("class", "total-val")
      .text(function(d, i) {
        var res = d3.format(that.format)(d.value);
        res = res.split(",").join(that.thousandSeparator);
        res = res.split(".").join(that.decimalSeparator);
        return res;
      })
      .attr("transform", function(d, i) {
        var dx = -x.range()[0] + (d.value > 0 ? x(d.value) : 0);
        return "translate(" + (dx) + "," + (y(d.title) + barWidth * 1.5) + ")";
      })
      .attr("x", "1em")
      .style("font-size", 11 / 15 + "em")
      .style("dominant-baseline", "central")
      .attr("fill", function(d){
        return d.value > 0 ? "#284b81" : "#e21a1a";
      });

  }
  svg.attr("width", width);


}


/*group chart--->*/

function GroupChart(options) {
  BasicChart.apply(this, arguments);
  this.base = new BasicChart(options);
  this.options = options;
  this.data = options.data;
  this.outerWidth = this.width - this.graphPadding.left - this.graphPadding.right;
  this.outerHeight = this.height - this.graphPadding.top - this.graphPadding.bottom;
  this.orientation = options.orientation || "horizontal";
  this.format = options.format === undefined ? ",.1f" : options.format;
  var graphPadding = {
    top: 20,
    left: 0,
    right: 50,
    bottom: 5
  };
  if (this.orientation == "vertical") {
    graphPadding.left = 15;
    graphPadding.right = 15;
  }
  this.graphPadding = options.graphPadding || graphPadding;
  this.graphHeight = this.height;
  var legendPadding = {
    top: 15,
    left: 0,
    right: 20,
    bottom: 5
  };
  if (this.orientation == "vertical") legendPadding.top = 40;
  this.legendPadding = options.legendPadding || legendPadding;
  this.bottomLinePadding = options.bottomLinePadding || {
    left: 0,
    right: 0
  };
  this.titles = this.data.layers.map(function(d) {
    return d.title
  });
  this.barWidth = options.barWidth === undefined ? 20 : options.barWidth;
  this.textHeight = 15;
  return this;
}

GroupChart.prototype.getColor = function(title) {
  // TODO: make this with ordinal scale
  var colors = this.colors;
  var titles = this.titles;
  var res = null;
  titles.forEach(function(t, i) {
    if (t == title)
      res = colors[i % colors.length];
    return;
  })
  return res;
}

GroupChart.prototype.showOnly = function(keys) {
  
  return;
  var duration = this.options.duration === undefined ? 600 : this.options.duration;
  var x = this.x;
  var y = this.y;
  var suby = this.suby;
  var that = this;
  var field = this.field;
  var keyIndex = this.titles.indexOf(keys[0]);
  var newGroupLayers = this.layersData.filter(function(d) {
    return keys.some(function(k) {
      return k == d.key
    })
  });
  var vals = newLayers.map(function(d) {
    return d.value
  });

  var totals = this.dates.map(function(d, i) {
    return {
      title: d,
      value: d3.sum(newLayers, function(l) {
        return l.value[i].y;
      })
    }
  })
  var totalShares = totals.map(function(d, i){
    var value = d.value/that.totals[i].value;
    return {
      title: d.title,
      value: value,
      absoluteValue: d.value
    }
  })
  // s
  


  if (that.options.orientation == "vertical") {
    y.domain([0, d3.max(d3.merge(vals), function(d) {
      return keys.length == 1 ? d.y : d.y0 + d.y;
    })]);
    var layersGroup = field.selectAll(".layer").data(newLayers, function(d) {
      return d.key
    });
    var exit = layersGroup.exit()
    exit.selectAll("rect")
      .transition()
      .duration(duration)
      .attr("transform", function(d, i) {
        var index = that.titles.indexOf(d.key);
        var dy = index > keyIndex ? y.range()[1] - y(totals[i].value) : y.range()[1];
        return "translate(" + (x(d.x)) + ", " + (dy) + ") scale(1, 0)";
      });

    var enter = layersGroup.enter()
    var update = layersGroup
    update.selectAll("rect")
      .transition()
      .duration(duration)
      .attr("height", function(d) {
        return y(d.y)
      })
      .attr("transform", function(d) {
        var dy = keys.length > 1 ? y.range()[1] - y(d.y0) - y(d.y) : y.range()[1]-y(d.y)
        return "translate(" + (x(d.x)) + ", " + (dy) + ")";
      });

    update.selectAll("text.label")
      .transition()
      .duration(duration)
      .attr("transform", function(d) {
        var dy = keys.length > 1 ? y(d.y0) + y(d.y) * .5 : y(d.y) * .5;
        return "translate(" + dx + ", " + (y(d.x) + y.rangeBand() * .5) + ")";
      })
      .attr("opacity", 1)

    field.selectAll(".total-val")
      .data(totals)
      .transition()
      .duration(duration)
      .attr("transform", function(d, i) {
        var dy = -y(d.value) + y.range()[1];
        return "translate(" + (x(d.title)) + "," + (dy) + ")";
      })
      .tween("fill", function(d, i) {
        var ip = d3.interpolateRgb(d3.select(this).attr("fill"), keys.length > 1 ? "#425563" : that.getColor(keys[0]));
        return function(t) {
          d3.select(this).attr("fill", ip(t));
        };
      })
      .tween("text", function(d, i) {
        var ip = d3.interpolateNumber(this.textContent.split(that.thousandSeparator).join("").split(that.decimalSeparator).join("."), d.value);
        return function(t) {
          res = ip(t);
          res = d3.format(that.format)(res);
          res = res.split(",").join(that.thousandSeparator);
          res = res.split(".").join(that.decimalSeparator);
          this.textContent = res;
        };
      })


  }
  else {
    if (that.options.type == "stack") {
      x.domain([0, d3.max(d3.merge(vals), function(d) {
        return keys.length == 1 ? d.y : d.y0 + d.y;
      })]);
      var layersGroup = field.selectAll(".layer").data(newLayers, function(d) {
        return d.key
      });
      var exit = layersGroup.exit()
      exit.selectAll("rect")
        .transition()
        .duration(duration)
        .attr("transform", function(d, i) {
          var index = that.titles.indexOf(d.key);
          var dx = index > keyIndex ? x(totals[i].value) : 0;
          return "translate(" + (dx) + ", " + (y(d.x) + that.barWidth) + ") scale(0, 1)";
        });

      var enter = layersGroup.enter()
      var update = layersGroup
      update.selectAll("rect")
        .transition()
        .duration(duration)
        .attr("width", function(d) {
          return x(d.y)
        })
        .attr("transform", function(d) {
          var dx = keys.length > 1 ? x(d.y0) : 0
          return "translate(" + dx + ", " + (y(d.x) + that.barWidth) + ")";
        });

      update.selectAll("text.label")
        .transition()
        .duration(duration)
        .attr("transform", function(d) {
          
          var dx = keys.length > 1 ? x(d.y0) + x(d.y) * .5 : x(d.y) * .5;
          return "translate(" + dx + ", " + (y(d.x) + y.rangeBand() * .5) + ")";
        })
        .attr("opacity", 1)

      field.selectAll(".total-val")
        .data(totals)
        .transition()
        .duration(duration)
        .attr("transform", function(d, i) {
          var dx = x(d.value);
          return "translate(" + (dx) + "," + (y(d.title) + that.barWidth * 1.5) + ")";
        })
        .tween("fill", function(d, i) {
          var ip = d3.interpolateRgb(d3.select(this).attr("fill"), keys.length > 1 ? "#425563" : that.getColor(keys[0]));
          return function(t) {
            d3.select(this).attr("fill", ip(t));
          };
        })
        .tween("text", function(d, i) {
          var ip = d3.interpolateNumber(this.textContent.split(that.thousandSeparator).join("").split(that.decimalSeparator).join("."), d.value);
          return function(t) {
            res = ip(t);
            res = d3.format(that.format)(res);
            res = res.split(",").join(that.thousandSeparator);
            res = res.split(".").join(that.decimalSeparator);
            this.textContent = res;
          };
        })
        if (that.showPercents) {
          var percents = field.select(".stack-graph").selectAll("text.percent-val").data(totalShares)
          // percents.exit().transition().attr("opacity", 0)
          percents.transition().attr("opacity", 0).transition().attr("opacity", function(d){
            return keys.length > 1 ? 0 : 1;
          }).text(function(d){
            var labelWidth = this.getBBox().width;
            var formatted = d3.format(".1%")(d.value);
            if (x(d.absoluteValue) < labelWidth+13) {
              formatted = "("+formatted+")";
            }
            return d.value == 0 ? "" : formatted;
          })
          .attr("transform", function(d,i){
            var labelWidth = this.getBBox().width;
            var dx = x.range()[0];
            if (x(d.absoluteValue) < labelWidth+13) {
              dx += labelWidth+x(d.absoluteValue)-10;
            }
            return "translate(" + (dx) + "," + (y(d.title) + y.rangeBand()*.75) + ")";
          }).attr("fill", function(d){
            var labelWidth = this.getBBox().width;
              return (x(d.absoluteValue) < labelWidth+13) ? "#000" : "#fff"
              
          })
          percents.enter().append("text").attr("class", "percent-val").attr("transform", function(d,i){
              return "translate(" + (x.range()[0]) + "," + (y(d.title) + y.rangeBand()*.75) + ")";
            })
          .style("font-size", 13 / 15 + "em")
          .attr("x", "1em")
          .text(function(d){
              return d3.format(".1%")(d.value);
            }).attr("fill", "#fff").attr("opacity", 0)
          .transition().attr("opacity", 1);
        }
        
    }

    if (that.options.type == "group") {
      x.domain([0, d3.max(d3.merge(vals), function(d) {
        return d.y;
      })]);
      // x.range([0, this.width]);
      var layersGroup = field.selectAll(".layer").data(newLayers, function(d) {
        return d.key
      });
      var exit = layersGroup.exit()
      exit.selectAll("rect")
        .transition()
      // .attr("height", 0)
      .attr("opacity", 0)
      // .attr("height", (y.rangeBand()/newLayers.length - 1))
      .attr("transform", function(d, i, j) {
        var dx = x(0);
        var dy = (y(d.x) + (j + 2) * y.rangeBand() / that.titles.length);
        return "translate(" + dx + ", " + dy + ")";
      });

      var enter = layersGroup.enter()

      var update = layersGroup
      update.selectAll("rect").transition()
        .attr("width", function(d) {

          return x(d.y)
        })
        .attr("transform", function(d, i, j) {
          var dx = x(0);
          var dy = (y(d.x) + j * y.rangeBand() / newLayers.length);
          return "translate(" + dx + ", " + dy + ")";
        })
        .attr("height", function(d) {
          return keys.length == 1 ? y.rangeBand() : (y.rangeBand() / that.titles.length - 1)
        })
        .attr("opacity", 1)

      update.selectAll("text.label")
        .transition()
        .attr("transform", function(d, i, j) {
          var dx = x(d.y) + 5;
          var dy = (y(d.x) + j * y.rangeBand() / newLayers.length) + y.rangeBand() / newLayers.length * .5;

          return "translate(" + dx + ", " + dy + ")";
        })
        .attr("opacity", 1)

      exit.selectAll("text.label")
        .transition()
        .attr("opacity", 0)
      // .attr("transform", function(d) {
      //  var dx = x(d.y)*.5
      //  return "translate(" + dx + ", " + (y(d.x)+y.rangeBand()*.5) + ")";
      // })
    }
  }


  
  var leg = this.legendItems.data(keys, function(d) {
    return d
  });
  var legEnter = leg.enter();
  var legExit = leg.exit();
  leg
    .transition()
    .attr("opacity", 1)
    .selectAll("rect")
    .attr("stroke-width", function() {
      return keys.length == 1 ? 2 : 0;
    })

  legExit
    .transition()
    .attr("opacity", .5)
    .selectAll("rect")
    .attr("stroke-width", 0)


}

GroupChart.prototype.drawVertical = function() {
  

  var that = this;
  var data = this.data;
  var stack = this.stack = d3.layout.stack();
  var dates = this.dates = this.data.categories;
  var padding = this.graphPadding;
  var width = this.width;
  var height = this.height;
  var offsetLeft = this.offsetLeft;
  var markHeight = 15; // legend mark height
  var sd = this.layers = this.data.layers.map(function(d, i) {
    return d.values.map(function(v, j) {
      return {
        x: dates[j],
        y: v
      };
    });
  })
  var layers = stack(sd);
  var svg = this.svg;
  var field = this.field;
  var header = this.header;
  var headerHeight = header.node().getBBox().height;
  var graphArea = this.graphArea
  var graph = graphArea.append("g").classed("stack-graph", true)
    .attr("transform", "translate(" + this.graphPadding.left + "," + this.graphPadding.top + ")");
  var gHeight = this.graphHeight - this.graphPadding.top - this.graphPadding.bottom;
  var gWidth = this.graphWidth - this.graphPadding.left - this.graphPadding.right;
  var x = this.x = d3.scale.ordinal();
  var y = this.y = d3.scale.linear();
  var max = d3.max(d3.merge(layers), function(d) {
    return d.y0 + d.y;
  });
  var min = d3.min(d3.merge(layers), function(d) {
    return d.y0 + d.y;
  });
  var maxDomain = max;
  var maxRange = gHeight;
  if (max*min < 0) {
    maxDomain = Math.max(max, -min);
    maxRange = gHeight*.5
  }
  y.domain([0, maxDomain]);
  
  y.range([0, maxRange])
  x.rangeBands([0, gWidth], .5, 0);
  x.domain(dates);

  var xAxis = graph.append("g").attr("class", "x axis")
  xAxis.selectAll(".tick").data(dates)
    .enter().append("text").attr("class", "tick")
    .text(String)
    .attr("transform", function(d) {
      
      return "translate("+x(d)+"," + (y.range()[1]+that.graphPadding.bottom) + ")";
    })
    .attr("y", "1.5em")
    .attr("x", x.rangeBand()*.5)
    .style("font-size", 12 / 15 + "em")
    .attr("text-anchor", "middle")
  // .style("dominant-baseline", "central")
  .attr("fill", "#284b81");

  var layersData = this.layersData = layers.map(function(d, i) {
    return {
      key: that.titles[i],
      value: d
    }
  })

  var layer = graph.selectAll("g.layer")
    .data(layersData, function(d, i) {
      return d.key;
    })
    .enter().append("svg:g")
    .attr("class", function(d, i) {
      return "layer-" + i
    })
    .classed("layer", true)
    .attr("fill", function(d) {
      return that.getColor(d.key);
    })
    .attr("stroke", function(d) {
      return that.getColor(d.key);
    })

    
    var rect = layer.selectAll("rect")
      .data(function(d) {
        return d.value.map(function(v) {
          return {
            x: v.x,
            y: v.y,
            y0: v.y0,
            key: d.key
          };
        });
      })
      .enter().append("svg:rect")
      .attr("height", function(d) {
        
        
        return Math.abs(y(d.y));
      })
      .attr("width", function(d) {
        return x.rangeBand();
      })
      .attr("transform", function(d) {
        var dy = y.range()[1] - (d.y+d.y0 > 0 ? y(d.y+d.y0) : 0)
        return "translate(" + x(d.x) + ", " + dy + ")";
      })
      .attr("stroke-width", 0);

      if (data.layers.length > 1) {
        rect.on("mouseover", function(d) {
          that.showOnly([d.key]);
        }).on("mouseout", function() {
          that.showOnly(that.titles)
        })
      }

    var totals, calcTotals;
    calcTotals = dates.map(function(d, i) {
      return {
        "title": d,
        "value": d3.sum(data.layers, function(l) {
          return l.values[i];
        })
      }
    })
    this.totals = totals = calcTotals;

    var totalText = graph.selectAll(".total-val").data(totals)
      .enter().append("text").attr("class", "total-val")
      .text(function(d, i) {
        var res = d3.format(that.format)(d.value);
        res = res.split(",").join(that.thousandSeparator);
        res = res.split(".").join(that.decimalSeparator);
        return res;
      })
      .attr("transform", function(d, i) {
        
        var dy = -y(d.value) + y.range()[1];
        if (d.value < 0) {
          xAxis.selectAll(".tick").filter(function(d,j){
            return j==i;
          }).attr("y", "-1.5em")
        }
        return "translate(" + (x(d.title)) + "," + (dy) + ")";
      })
      .attr("x", x.rangeBand()*.5)
      .attr("y", function(d){
        return d.value > 0 ? "-.5em" : "1.5em"
      })
      .attr("text-anchor", "middle")
      .style("font-size", 11 / 15 + "em")
      // .style("dominant-baseline", "central")
      .attr("fill", function(d){
        return d.value < 0 ? "#e21a1a" : "#284b81";
      });
}

GroupChart.prototype.draw = function() {
  // TODO: stack chart with negative values (multiple)
  this.base.drawBase.call(this);

  if (this.options.orientation == "vertical")  {
    this.drawVertical();
    return;
  }
  var that = this;
  var data = this.data;
  var dates = this.dates = this.data.categories;
  var padding = this.graphPadding;
  var width = this.width;
  var height = this.height;
  var offsetLeft = this.offsetLeft;
  var markHeight = 15; // legend mark height
  
  var svg = this.svg;

  var field = this.field;

  var header = this.header;
  var headerHeight = header.node().getBBox().height;
  var graphArea = this.graphArea
  var graph = graphArea.append("g").classed("stack-graph", true)
    .attr("transform", "translate(" + this.graphPadding.left + "," + (this.graphPadding.top) + ")");
  var gHeight = this.graphHeight - this.graphPadding.top - this.graphPadding.bottom;

  var x = this.x = d3.scale.linear();
  var gWidth = width - this.graphPadding.left - this.graphPadding.right;
  var y = this.y = d3.scale.ordinal().rangeBands([0, gHeight], 0, 0);
  y.domain(dates);

  var titleHeight = 15;
  var barWidth = this.barWidth = this.y.rangeBand() * .45;
  var allValues = d3.merge(d3.merge(this.data.layers).map(get("values")));

  var max = d3.max(allValues)
  var min = d3.min(allValues);
  var maxDomain = max;
  var minDomain = 0;
  var minRange = 0;
  var maxRange = gWidth;
  if (max*min < 0) {
    maxDomain = Math.max(max, -min);
    minDomain = -maxDomain;
    // maxRange = gWidth*.5;
    minRange = -gWidth*.5
  }
  x.range([minRange, minRange + maxRange]);
  x.domain([0, max]);
  var suby = this.suby = d3.scale.ordinal().rangeBands([titleHeight, y.rangeBand()], .1, .2).domain(this.titles);

  var yAxis = graph.append("g").attr("class", "y axis")
  yAxis.selectAll(".tick").data(dates)
    .enter().append("text").attr("class", "tick")
    .text(String)
    .attr("transform", function(d) {
      return "translate(0," + (y(d)) + ")";
    })
    .attr("y", "1em")
    .style("font-size", 12 / 15 + "em")
  // .style("dominant-baseline", "central")
  .attr("fill", "#284b81");

  

  // Add a rect for each date.
 

  var groupLayers = this.data.layers.map(function(l)  {
    return {
      title: l.title,
      values: l.values.map(function(v,i){
        return {
          value: v,
          key: dates[i]
        }
      })
    }
  })
  var layer = graph.selectAll("g.layer")
    .data(groupLayers, function(d, i) {
      return d.title;
    })
    .enter().append("svg:g")
    .attr("class", function(d, i) {
      return "layer-" + i
    })
    .classed("layer", true)
    .attr("fill", function(d) {
      return that.getColor(d.title);
    })
    .attr("stroke", function(d) {
      return that.getColor(d.title);
    }).attr("transform", function(d){
      
      var dy = suby(d.title);
      // dy = suby.range()[0];
      return "translate(0,"+dy+")"
    })

  var rect = layer.selectAll("rect")
    .data(function(d){
      
      return d.values;
    })
    .enter().append("svg:rect")
    .attr("height", suby.rangeBand())
    .attr("width", function(d) {
      return x(d.value);
    })
    .attr("transform", function(d, i, j) {
      var dx = 0;
      var dy = (y(d.key));
      return "translate(" + dx + ", " + dy + ")";
    })
    .attr("stroke-width", 0);
  var maxWidth = 0;

  var label = layer.selectAll("text.label")
    .data(function(d) {
      
      return d.values
    })
    .enter().append("text")
    .attr("class", "label")
    .attr("transform", function(d, i, j) {
      var dx = x(d.value) + 5;
      var dy = y(d.key) + suby.rangeBand()*.5;
      return "translate(" + dx + ", " + dy + ")";
    })
    .attr("stroke-width", 0)
    .attr("font-size", 10 / 15 + "em")
    .text(function(d) {
      var res = d3.format(that.format)(d.value);
      res = res.split(",").join(that.thousandSeparator);
      res = res.split(".").join(that.decimalSeparator);
      return res;
    })
    .style("dominant-baseline", "central")
    .attr("text-anchor", "start")
  
  if (data.layers.length > 1) {
    layer.on("mouseover", function(d) {
      that.showOnly([d.title]);
    }).on("mouseout", function() {
      that.showOnly(that.titles)
    })
  }
  //  adding total values
  var maxWidth = 0;
  svg.attr("width", width);


}


/*<---group chart*/

function PieChart(options) {
  BasicChart.apply(this, arguments);
  this.base = new BasicChart(options);
  this.sorting = options.sorting || "descending";
  this.outerWidth = this.width - this.graphPadding.left - this.graphPadding.right;
  this.outerHeight = this.height - this.graphPadding.top - this.graphPadding.bottom;
  this.graphHeight = Math.min(this.outerWidth, this.outerHeight) + this.graphPadding.top + this.graphPadding.bottom;
  this.markHeight = 15;
  this.titles = this.base.data.layers.map(function(d) {
    return d.title
  });
  this.centerColor = options.centerColor === undefined ? "#d55949" : options.centerColor;
  this.prefix = options.prefix === undefined ? "" : options.prefix;
  this.format = options.format === undefined ? ",.1%" : options.format;
  this.totalFormat = options.totalFormat === undefined ? ",.1f" : options.totalFormat;
  this.centerFontSize = options.centerFontSize === undefined ? 12 : options.centerFontSize;
  this.legendPadding = options.legendPadding || {
    top: 10,
    left: 10,
    right: 20,
    bottom: 5
  };
  return this;
}

PieChart.prototype.getColor = function(title) {
  var colors = this.colors;
  var titles = this.titles;
  var res = null;
  titles.forEach(function(t, i) {
    if (t == title) res = colors[i % colors.length];
    return;
  })
  return res;
}

PieChart.prototype.showOnly = function(keys) {
  var r = this.r;
  var that = this;
  var newLayers = this.data.layers.filter(function(d) {
    return keys.some(function(k) {
      return k == d.title
    })
  })
  var update = this.field.selectAll(".arc").data(this.pie(newLayers), function(d) {
    return d.data.title
  });
  var exit = update.exit();

  update.selectAll("path").transition().attr("transform", function(d) {
    var angle = .5 * (d.endAngle + d.startAngle) - Math.PI * .5;
    return keys.length == 1 ? "translate(" + (-Math.cos(angle) * r * .5) + "," + (-Math.sin(angle) * r * .5) + ") scale(.5) translate(" + 2 * Math.cos(angle) * r * .5 + "," + 2 * Math.sin(angle) * r * .5 + ") scale(2.1)" : "";
  })

  var curStart = 0;
  update.selectAll("text")
    .transition()
    .attr("opacity", function(d) {
      return keys.length == 1 && d.data.title == keys[0] ? 1 : 0;
    })
  exit.selectAll("text")
    .transition()
    .attr("opacity", 0)

  var radius = this.r;
  var centerText = this.graphArea.select(".chart-sum-text");
  centerText.transition().duration(300)
    .tween("fill", function(d, i) {
      var ip = d3.interpolateRgb(d3.select(this).attr("fill"), keys.length > 1 ? that.centerColor : that.getColor(keys[0]));
      return function(t) {
        d3.select(this).attr("fill", ip(t));
      };
    })
  var sum = keys.length == 1 ? newLayers[0].value : this.sum;
  var res;
  res = [sum];
  res.push(this.data.units);
  var labels = centerText.data(res)
    .selectAll("tspan").data(function(d) {
      return [d];
    })
  labels.transition().duration(300).tween("text", function(d, i) {
    var ip = d3.interpolateNumber(this.textContent.split(that.thousandSeparator).join("").split(that.decimalSeparator).join(".").replace(that.prefix, ""), d);
    return function(t) {
      res = ip(t);
      
      
      res = d3.format(that.totalFormat)(res);
      
      res = res.split(",").join(that.thousandSeparator);
      res = res.split(".").join(that.decimalSeparator);
      res = that.prefix.toString() + res;
      
      this.textContent = res;
    };
  })
  centerText.attr("transform", function(d) {
    return "translate(" + (radius + that.graphPadding.left) + "," + (radius + that.graphPadding.top) + ")";
  })


  var leg = this.legendItems.data(keys, function(d) {
    return d
  });
  var legEnter = leg.enter();
  var legExit = leg.exit();
  leg
    .transition()
    .duration(300)
    .attr("opacity", 1)

  legExit
    .transition()
    .duration(300)
    .attr("opacity", .5)

}



PieChart.prototype.draw = function() {

  this.base.drawBase.call(this);
  var that = this;
  var data = this.data;
  var width = this.width,
    height = this.height;
  var field = this.field
  if (that.sorting) {
    data.layers.sort(function(a, b) {
      return d3[that.sorting](a.value, b.value);
    })
  }
  this.titles = data.layers.map(function(d) {
    return d.title
  });
  var pie = this.pie = d3.layout.pie()
    .sort(function(a, b) {
      if (that.sorting == "descending") {
        return d3.descending(a.value, b.value);
      } else if (that.sorting == "ascending") {
        return d3.ascending(a.value, b.value);
      } else {
        return 0;
      }
    })
    .value(function(d) {
      return parseFloat(d.value);
    })

  var arc = this.arc = d3.svg.arc();

  var svg = this.svg;

  var outerWidth = this.outerWidth;
  var outerHeight = this.outerHeight;

  var radius = (Math.min(outerWidth, outerHeight)) * .5;

  arc.outerRadius(radius)
    .innerRadius(radius * .725);

  var values = this.values = data.layers.map(function(d) {
    return parseFloat(d.value);
  });

  var startPieData = pie(data.layers).map(function(d) {
    return {
      data: d.data,
      endAngle: d.endAngle,
      startAngle: d.startAngle,
      value: d.value
    }
  })
  var graphArea = this.graphArea;
  var g = this.graphArea
    .append("g").classed("pie-circle", true)
    .attr("transform", "translate(" + this.graphPadding.left + "," + this.graphPadding.top + ")")
    .selectAll(".arc")
    .data(startPieData)
    .enter().append("g")
    .attr("class", "arc")
    .attr("transform", "translate(" + (radius) + "," + (radius) + ")");

  var path = this.path = g.append("path")
    .attr("d", arc)
    .each(function(d) {
      this._current = d;
    })
    .attr("fill", function(d) {

      return that.getColor(d.data.title);
    })
    .attr("stroke", "none")

  path = path.data(pie(data.layers), function(d) {
    return d.data.title
  }); // update the data
  path
    .transition().delay(0).duration(0).attrTween("d", arcTween)
    .each("end", function() {

      path.attr("transform", function(d) {
        var angle = .5 * (d.endAngle + d.startAngle) - Math.PI * .5;

        return "scale(1) translate(" + Math.cos(angle) * r * .005 + "," + Math.sin(angle) * r * .005 + ")";
      });
      path.on("mouseover", function(d) {
        that.showOnly([d.data.title])
      })
        .on("mouseout", function(d) {

          that.showOnly(that.titles);
        })
    }); // redraw the arcs

  function arcTween(a) {
    var i = d3.interpolate(this._current, a);
    this._current = i(0);
    return function(t) {
      return arc(i(t));
    };
  }


  var sum = this.sum = data.layers.map(function(a) {
    return parseFloat(a.value)
  }).reduce(function(a, b) {
    return a + b;
  });
  var res;
  res = [sum];
  res.push(data.units);
  var sumText = graphArea.append("g")
    .attr("class", "chart-sum-text")
    .attr("transform", "translate(" + (radius) + "," + (radius) + ")")
    .attr("text-anchor", "middle")
    .attr("fill", that.centerColor)
    .attr("font-size", that.centerFontSize)
    .style("dominant-baseline", "central")
  sumText.append("text").selectAll("tspan").data(res)
    .enter()
    .append("tspan").text(function(d, i) {
      if (i == 0) {
        d = d3.format(that.totalFormat)(d);
        d = d.split(",").join(that.thousandSeparator);
        d = d.split(".").join(that.decimalSeparator);
        d = that.prefix.toString() + d.toString();
      }
      return d;
    }).attr("dy", function(d, i) {
      return i == 0 ? 0 : "1.33em";
    }).attr("x", 0)
    .attr("font-size", function(d, i) {
      return i == 0 ? "1em" : ".5em";
    })
  sumText.attr("transform", function(d) {
    return "translate(" + (radius + that.graphPadding.left) + "," + (radius + that.graphPadding.top) + ")";
  })
  // sumText.append("text").text(data.units).attr("dy",15).attr("font-size",11/15+"em")
  var r = this.r = radius;
  var pieData = this.pieData = pie(data.layers);

  var pieGroups = [];
  var curGroup = 0;
  var placingLimit = this.placingLimit = .0455;
  pieGroups[curGroup] = [];
  pieData.forEach(function(d, i) {
    var delta = Math.abs(d.startAngle - d.endAngle) / (2 * Math.PI);
    var next = pieData[(i + 1) % pieData.length];
    var nextDelta = Math.abs(next.startAngle - next.endAngle) / (2 * Math.PI);
    if (delta < placingLimit) {
      pieGroups[curGroup].push(i);
    } else if (nextDelta < placingLimit) {
      pieGroups[++curGroup] = [];
    }

  });
  // pieGroups.splice(-1)
  pieGroups = pieGroups.filter(function(d) {
    return d.length > 1
  });

  var pieGroupsData = pieGroups.map(function(pg) {
    return pg.map(function(i) {
      return pieData[i];
    })
  })
  var minDist = 16;
  var deltaAngle = minDist / this.r;
  var pieGroupsAngles = pieGroupsData.map(function(pg) {
    var sa = pg[0].startAngle;
    var ea = pg[pg.length - 1].endAngle;
    var ca = (sa + ea) / 2
    return {
      startAngle: sa,
      endAngle: ea,
      centerAngle: ca,
      data: pg,
      newStartAngle: ca - deltaAngle * pg.length * .5,
      newEndAngle: ca + deltaAngle * pg.length * .5,
    };
  })


  pieGroups.forEach(function(pg, i) {
    var pga = pieGroupsAngles[i];
    pg.forEach(function(p, j) {
      var pd = pieData[p];
      pd.newStartAngle = pga.newStartAngle + deltaAngle * j;
      pd.newEndAngle = pd.newStartAngle + deltaAngle;
    })
  })

  var outerLabelRadius = this.outerLabelRadius = r * 1.26;
  var innerLabelRadius = this.innerLabelRadius = r * 1.1;
  var curStart = 0;
  var labels = g.append("text")
    .data(pieData)
    .attr("fill", function(d) {
      return that.getColor(d.data.title);
    })
    .attr("transform", function(d, i) {
      return "translate(" + (Math.cos(((d.startAngle + d.endAngle - Math.PI) / 2)) * (r)) + "," + (Math.sin((d.startAngle + d.endAngle - Math.PI) / 2) * (r)) + ")";
    })
    .attr("x", function(d) {
      return 2 * Math.cos(((d.startAngle + d.endAngle - Math.PI) / 2)) + "em";
    })
    .attr("y", function(d) {
      return 2 * Math.sin(((d.startAngle + d.endAngle - Math.PI) / 2)) + "em";
    })
    .style("dominant-baseline", "central")
    .attr("text-anchor", "middle")
    .text(function(d) {
      var res = d3.format(that.format)(d.data.value / sum);
      res = res.split(".").join(that.decimalSeparator);
      return res;
    })
    .attr("opacity", function(d) {
      return d.data.value == 0 ? 0 : .0;
    })
    .style("font-size", function(d) {
      var delta = d.data.value / sum;
      var maxSize = 1.1;
      var minSize = .6;
      var size = delta > .5 ? maxSize : minSize + (delta + .5) * (maxSize - minSize)
      return size + "em";
    });

}


function buildChart(options) {
  var chart;
  var $cont = $(options.container);
  var name = options.container.slice(1);
  var fullRootPath = $('#site-title').attr('href');
  var rootPath = fullRootPath.substring(0, fullRootPath.length - 9);
  var path = rootPath+"img/chart/";
  var lang = $("body").attr("data-siteid");
  if (lang == "en") {
    name = name.slice(0, -3);
  }
  path += lang + "/";
  if ($cont.length == 0) return;
  options.container = $("<div>").appendTo($cont).addClass("chartcontent")
  $cont.append('<div class="footer">'+
    '<span class="mico zoom"></span>'+
    '<a href="'+path+'big/'+name+'.png" target="_blank" class="mico pngload" title="' + window.downpngchart + '"></a>'+
    '<a href="'+rootPath+'xls/'+'ru'+'/'+name+'.xlsx" target="_blank" class="mico xls" title="' + window.downxlstable + '"></a>'+
  '</div>')
  if (!supportSVG) {
    $cont.addClass("grafik-box");
    $("<p>").addClass("title").prependTo($cont).html(options.data.chartTitle);
    $cont.addClass("no-svg");
    $("<img>").appendTo(options.container).attr("src", path + name + ".png")
    return;
  } else {
    $cont.addClass("with-svg");
    switch (options.type) {
      case "stack":
        chart = new StackChart(options);
        break;
      case "group":
        chart = new GroupChart(options);
        break;
      case "pie":
        chart = new PieChart(options);
        break;
      case "line":
        chart = new LineChart(options);
        break;
      case "multiPie":
        chart = new MultiPieChart(options);
        break;
      case "passives":
        chart = new PassivesChart(options);
        break;
      case "barline":
        chart = new BarLineChart(options);
        break;
      case "barline2":
        chart = new BarLineDoubleChart(options);
        break;
      case "stock":
        chart = new StockChart(options);
        break;
      case "change":
        chart = new BarChangeChart(options)
    }

    chart.draw();

  }
  return;

}

function addPngLink(container, id) {
  container.find(".png-link").remove();
  $("<a/>").appendTo(container.find(".chart-links"))
    .attr("href", "/images/charts/" + id.slice(1) + ".png")
    .attr("download", id.slice(1) + ".png")
    .addClass("png-link").html("png")
}

function saveToSVG(container) {
  //Encode the SVG
  if (!window.btoa) return;
  var serializer = new XMLSerializer();
  var xmlString = serializer.serializeToString(d3.select(container).select("svg").node());
  var imgData = 'data:image/svg+xml;base64,' + utf8_to_b64(xmlString);
  //Use the download attribute (or a shim) to provide a link
  d3.select(container).select(".chart-links").append("a")
    .attr("href", imgData).attr("download", container.slice(1) + ".svg").html("svg")
    .attr("class", "svg-link");
}

function utf8_to_b64(str) {
  return window.btoa(unescape(encodeURIComponent(str)));
}

function checkSVG() {
  var ns = {
    'svg': 'http://www.w3.org/2000/svg'
  };
  var hasSvg = function() {
    return !!document.createElementNS && !! document.createElementNS(ns.svg, 'svg').createSVGRect;
  };

  return hasSvg();
}